// Copyright 2019 Diffuse Logic, all rights reserved.

#ifndef DL_CORE_LOGGING_H
#define DL_CORE_LOGGING_H

#include "dl_string.h"

namespace dl {

enum LogType
{
    LogType_Info,
    LogType_Warning,
    LogType_Error,
    LogType_Custom
};

// Callback for logging functions.
//
typedef void( *pLogFunc )( void*, LogType, const char* );

// Subscribe to be called back for logging.
//
DL_API void     subscribeLog( void* ctx, pLogFunc func );
DL_API void     unsubscribeLog( void* ctx );

DL_API void     openLogFile( String path = "" );
DL_API void     closeLogFile();

// Call this when your console is ready to print messages.
//
DL_API void      flushStartupMessages();

// Call these at any time.
//
DL_API void     logMessage( LogType type, String msg );
DL_API void     logMessage( LogType type, const char* fmt, ... );
DL_API void     logMessagev( LogType type, const char* fmt, va_list ap );
DL_API void     logMessageTsv( LogType type, const char* fmt, va_list ap ); // Add timestamp

DL_API void     logInfo( const char* fmt, ... );
DL_API void     logWarning( const char* fmt, ... );
DL_API void     logError( const char* fmt, ... );
DL_API void     logCustom( const char* fmt, ... );

DL_API void     logInfoTs( const char* fmt, ... );
DL_API void     logWarningTs( const char* fmt, ... );
DL_API void     logErrorTs( const char* fmt, ... );
DL_API void     logCustomTs( const char* fmt, ... );

DL_API void     logNewLine();
DL_API void     logSeparator( );
DL_API void     logBanner( const char* fmt, ... );

// Console color.
//
enum ConsoleColor
{
    ConsoleColor_Default,
    ConsoleColor_Red,
    ConsoleColor_Yellow,
    ConsoleColor_Cyan
};

DL_API void     setConsoleColor( ConsoleColor color );
DL_API void     resetConsoleColor();

#define DL_PRINT_COLOR( CONSOLE_COLOR, FMT, ... ) \
    dl::setConsoleColor( CONSOLE_COLOR ); \
    fprintf( stdout, FMT, __VA_ARGS__ ); \
    dl::resetConsoleColor();

#define DL_PRINT( FMT, ... ) \
    DL_PRINT_COLOR( dl::ConsoleColor_Default, FMT, __VA_ARGS__ )

#define DL_PRINT_RED( FMT, ... ) \
    DL_PRINT_COLOR( dl::ConsoleColor_Red, FMT, __VA_ARGS__ )

#define DL_PRINT_YELLOW( FMT, ... ) \
    DL_PRINT_COLOR( dl::ConsoleColor_Yellow, FMT, __VA_ARGS__ )

#define DL_PRINT_CYAN( FMT, ... ) \
    DL_PRINT_COLOR( dl::ConsoleColor_Cyan, FMT, __VA_ARGS__ )

} // dl

#endif // DL_CORE_LOGGING_H
